package nachos.threads;

import nachos.machine.*;

/**
 * Uses the hardware timer to provide preemption, and to allow threads to sleep
 * until a certain time.
 */
public class Alarm {
    /**
     * Allocate a new Alarm. Set the machine's timer interrupt handler to this
     * alarm's callback.
     *
     * <p><b>Note</b>: Nachos will not function correctly with more than one
     * alarm.
     */
    public Alarm() {
	Machine.timer().setInterruptHandler(new Runnable() {
		public void run() { timerInterrupt(); }
	    });
    }

    /**
     * The timer interrupt handler. This is called by the machine's timer
     * periodically (approximately every 500 clock ticks). Causes the current
     * thread to yield, forcing a context switch if there is another thread
     * that should be run.
     */
    public void timerInterrupt() {
	Lib.debug(dbgAlarm,"In Interrupt Handler (time = "+Machine.timer().getTime()+")");
	KThread.currentThread().yield();

/*------------------------------------Jade's------------------------------------*/
        final boolean intStatus = Machine.interrupt().disable();

        final long now = Machine.timer().getTime();
        for (int i = 0; i < pairs.size(); i++) {
            final Pair pair = this.pairs.get(i);
            if (now >= pair.timeout) {
                pair.semaphore.V();
                this.pairs.remove(i--);
            }
        }

        Machine.interrupt().restore(intStatus);
/*------------------------------------Jade's------------------------------------*/
    }

    /**
     * Put the current thread to sleep for at least <i>x</i> ticks,
     * waking it up in the timer interrupt handler. The thread must be
     * woken up (placed in the scheduler ready set) during the first timer
     * interrupt where
     *
     * <p><blockquote>
     * (current time) >= (WaitUntil called time)+(x)
     * </blockquote>
     *
     * @param	x	the minimum number of clock ticks to wait.
     *
     * @see	nachos.machine.Timer#getTime()
     */
    public void waitUntil(long x) {
	// This is a bad busy waiting solution 
	// long wakeTime = Machine.timer().getTime() + x;
	// while (wakeTime > Machine.timer().getTime())
	//    KThread.yield();

/*------------------------------------Jade's------------------------------------*/
        final boolean intStatus = Machine.interrupt().disable();

        final Pair pair = new Pair(Machine.timer().getTime() + x);
        this.pairs.add(pair);

        Machine.interrupt().restore(intStatus);

        pair.semaphore.P();
    }

    private static final class Pair {
        public final Semaphore semaphore = new Semaphore(0);
        public final long timeout;

        public Pair(final long timeout) {
            this.timeout = timeout;
        }
    }

    private final java.util.ArrayList<Pair> pairs = new java.util.ArrayList<Pair>();
/*------------------------------------Jade's------------------------------------*/

    /**
     * Tests whether this module is working.
     */
    public static void selfTest() {
	AlarmTest.runTest();
    }

    private static final char dbgAlarm = 'a';

}
