package nachos.threads;

import nachos.machine.*;

import java.util.LinkedList;

/**
 * An implementation of condition variables that disables interrupt()s for
 * synchronization.
 *
 * <p>
 * You must implement this.
 *
 * @see	nachos.threads.Condition
 */
public class Condition2 {
    /**
     * Allocate a new condition variable.
     *
     * @param	conditionLock	the lock associated with this condition
     *				variable. The current thread must hold this
     *				lock whenever it uses <tt>sleep()</tt>,
     *				<tt>wake()</tt>, or <tt>wakeAll()</tt>.
     */
    public Condition2(Lock conditionLock) {
        this.conditionLock = conditionLock;
    }

    /**
     * Atomically release the associated lock and go to sleep on this condition
     * variable until another thread wakes it using <tt>wake()</tt>. The
     * current thread must hold the associated lock. The thread will
     * automatically reacquire the lock before <tt>sleep()</tt> returns.
     */
    public void sleep() {

        /* If the current thread doesn't hold the lock, then abort */
        Lib.assertTrue(conditionLock.isHeldByCurrentThread());

/*------------------------------------Jade's------------------------------------*/
        conditionLock.release();

        final boolean intStatus = Machine.interrupt().disable();
        waitQueue.waitForAccess(KThread.currentThread());
        KThread.sleep();
        Machine.interrupt().restore(intStatus);

        conditionLock.acquire();
/*------------------------------------Jade's------------------------------------*/

    }

    /**
     * Wake up at most one thread sleeping on this condition variable. The
     * current thread must hold the associated lock.
     */
    public void wake() {
	Lib.assertTrue(conditionLock.isHeldByCurrentThread());
        
/*------------------------------------Jade's------------------------------------*/
        final boolean intStatus = Machine.interrupt().disable();

        final KThread thread = waitQueue.nextThread();
        if (null != thread)
            thread.ready();

        Machine.interrupt().restore(intStatus);
/*------------------------------------Jade's------------------------------------*/
    }

    /**
     * Wake up all threads sleeping on this condition variable. The current
     * thread must hold the associated lock.
     */
    public void wakeAll() {
	Lib.assertTrue(conditionLock.isHeldByCurrentThread());

/*------------------------------------Jade's------------------------------------*/
        final boolean intStatus = Machine.interrupt().disable();

        KThread thread;
        while (null != (thread = waitQueue.nextThread()))
            thread.ready();

        Machine.interrupt().restore(intStatus);
/*------------------------------------Jade's------------------------------------*/

    }

    /**
     * Tests whether this module is working.
     */
    public static void selfTest() {
        Condition2Test.runTest();
    }

    private static final char dbgCondition = 'c';

    private Lock conditionLock;

/*------------------------------------Jade's------------------------------------*/
    private final ThreadQueue waitQueue = ThreadedKernel.scheduler.newThreadQueue(false);
/*------------------------------------Jade's------------------------------------*/
}
